
#include <MixKeypad.h>

BleKeyboard bleKeyboard;

//keyboard matrix row column define
const byte ROWS = 3; //three rows
const byte COLS = 4; //four columns
//define the symbols on the buttons of the keypads
char hexaKeys[ROWS][COLS] = {
  {'0','1','2','3'},
  {'4','5','6','7'},
  {'8','9','A','B'}
};
byte rowPins[ROWS] = {14, 27, 26}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {25, 33, 32, 13}; //connect to the column pinouts of the keypad

//initialize an instance of class NewKeypad
Keypad customKeypad = Keypad( makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS); 

//assign single button pins for opt, shift, cmd
const int PIN15 = 15;
const int PIN02 = 02;
const int PIN00 = 00;

//asign single button pins for the other keys
const int PIN04 = 04;
const int PIN05 = 05;
const int PIN18 = 18;
const int PIN12 = 12;

//Variables will change:
int buttonPushCounterSHIFT = 0;  // counter for the number of button
int buttonStateSHIFT = 0;        // current state of the button
int lastButtonStateSHIFT = 0;    // previous state of the button

int buttonPushCounterCMD = 0;  // counter for the number of button
int buttonStateCMD = 0;        // current state of the button
int lastButtonStateCMD = 0;    // previous state of the button

int buttonPushCounterOPT = 0;  // counter for the number of button
int buttonStateOPT = 0;        // current state of the button
int lastButtonStateOPT = 0;    // previous state of the button


void print_wakeup_reason(){
  esp_sleep_wakeup_cause_t wakeup_reason;

  wakeup_reason = esp_sleep_get_wakeup_cause();

  switch(wakeup_reason)
  {
    case ESP_SLEEP_WAKEUP_EXT0 : Serial.println("Wakeup caused by external signal using RTC_IO"); break;
    case ESP_SLEEP_WAKEUP_EXT1 : Serial.println("Wakeup caused by external signal using RTC_CNTL"); break;
    case ESP_SLEEP_WAKEUP_TIMER : Serial.println("Wakeup caused by timer"); break;
    case ESP_SLEEP_WAKEUP_TOUCHPAD : Serial.println("Wakeup caused by touchpad"); break;
    case ESP_SLEEP_WAKEUP_ULP : Serial.println("Wakeup caused by ULP program"); break;
    default : Serial.printf("Wakeup was not caused by deep sleep: %d\n",wakeup_reason); break;
  }
}

void IRAM_ATTR isr(){
 while(digitalRead(PIN12) == LOW ){
  //Do nothing
 }
 detachInterrupt(digitalPinToInterrupt(12)); //because later used for wake up
  Serial.println("Going to sleep now");
  esp_sleep_enable_ext0_wakeup(GPIO_NUM_12,0); //1 = High, 0 = Low connected to GPIO12
  esp_deep_sleep_start(); 
 }


void setup() {

  esp_sleep_disable_wakeup_source(ESP_SLEEP_WAKEUP_ALL);
  // disable all wakeup sorces:
  pinMode (PIN12, INPUT_PULLUP); //pin 34-39 do not offer internal resistors; This will only work if RTC peripherals
  //are on in sleep, otherwise use 10kOhm external resistor for pull down
  

  Serial.begin(115200);
  delay(1000);
  Serial.println("Wake up!");
      //Print the wakeup reason for ESP32
  print_wakeup_reason();
  attachInterrupt(digitalPinToInterrupt(12), isr, FALLING);
  
  Serial.println("Starting BLE work!");
  bleKeyboard.begin();
  pinMode (PIN15, INPUT_PULLUP);
  pinMode (PIN02, INPUT_PULLUP);
  pinMode (PIN00, INPUT_PULLUP);
  pinMode (PIN04, INPUT_PULLUP);
  pinMode (PIN05, INPUT_PULLUP);
  pinMode (PIN18, INPUT_PULLUP);


boolean running = true;

}
  
void loop(){

  
  if(bleKeyboard.isConnected()) {

//12 button keyboard matrix
  {
    char customKey = customKeypad.getKey();
  
    if (customKey){
      switch(customKey){
        case '2':
            Serial.println("Sending 'j'");
            bleKeyboard.write('j');
            delay(50);
            break;
        case '3':
            Serial.println("Sending 'y'");
            bleKeyboard.write('y');
            delay(100);
            break;
        case '6':
            Serial.println("Sending 'z'");
            bleKeyboard.write('z');
            delay(50);
            break;
        case '7':
            Serial.println("Sending 'd'");
            bleKeyboard.write('d');
            delay(50);
            break;
        case '1':
            Serial.println("Sending 'b'");
            bleKeyboard.write('b');
            delay(50);
            break;
        case '5':
            Serial.println("Sending 'e'");
            bleKeyboard.write('e');
            delay(50);
            break;
        case '0':
            Serial.println("Sending 'p'");
            bleKeyboard.write('p');
            delay(50);
            break;
        case '4':
            Serial.println("Sending 'g'");
            bleKeyboard.write('g');
            delay(50);
            break;        
        case '9':
            Serial.println("Sending 'm'");
            bleKeyboard.write('m');
            delay(50);
            break;
        case 'B':
            Serial.println("Sending 'i'");
            bleKeyboard.write('i');
            delay(50);
            break;
        case '8':
            Serial.println("Sending 'w'");
            bleKeyboard.write('w');
            delay(50);
            break;
        case 'A':
            Serial.println("Sending 'r'");
            bleKeyboard.write('r');
            delay(50);
            break;                
        }
      }
    }

//opt, shift, cmd
buttonStateSHIFT = digitalRead(PIN02);
  if (buttonStateSHIFT != lastButtonStateSHIFT) {
    if (buttonStateSHIFT == LOW ){
      buttonPushCounterSHIFT++;
      Serial.println("Pressing 'LeftShift'");
      bleKeyboard.press(KEY_LEFT_SHIFT);
      Serial.print("number of button pushes: ");
      Serial.println(buttonPushCounterSHIFT);
    } else{
      Serial.println("releaseing 'LEFTSHIFT'");
      bleKeyboard.release(KEY_LEFT_SHIFT);
    }
    delay(50);
  }

lastButtonStateSHIFT = buttonStateSHIFT;


buttonStateCMD = digitalRead(PIN15);
  if (buttonStateCMD != lastButtonStateCMD) {
    if (buttonStateCMD == LOW ){
      buttonPushCounterCMD++;
      Serial.println("Pressing 'LeftCMD'");
      bleKeyboard.press(KEY_LEFT_GUI);
      Serial.print("number of button pushes: ");
      Serial.println(buttonPushCounterCMD);
    } else{
      Serial.println("releaseing 'LEFTCMD'");
      bleKeyboard.release(KEY_LEFT_GUI);
    }
    delay(50);
  }

lastButtonStateCMD = buttonStateCMD;


buttonStateOPT = digitalRead(PIN00);
  if (buttonStateOPT != lastButtonStateOPT) {
    if (buttonStateOPT == LOW ){
      buttonPushCounterOPT++;
      Serial.println("Pressing 'OPT'");
      bleKeyboard.press(KEY_LEFT_ALT);
      Serial.print("number of button pushes: ");
      Serial.println(buttonPushCounterOPT);
    } else{
      Serial.println("releaseing 'OPT'");
      bleKeyboard.release(KEY_LEFT_ALT);
    }
    delay(50);
  }

lastButtonStateOPT = buttonStateOPT;


//other key assign
  if(digitalRead(PIN04) == LOW ){
    Serial.println("Sending '['");
    bleKeyboard.write('[');
    delay(200);
  }

      if(digitalRead(PIN05) == LOW ){
    Serial.println("Sending ']'");
    bleKeyboard.write(']');
    delay(200);
   }

  if(digitalRead(PIN18) == LOW ){
    Serial.println("Sending 'Cmd+S'");
    bleKeyboard.press(KEY_LEFT_GUI);
    bleKeyboard.write('s');
    delay(100);
    bleKeyboard.release(KEY_LEFT_GUI);
    delay(200);
   }


}

}
